@@ -1075,9 +1075,9 @@ int _GMP_is_frobenius_underwood_pseudoprime(mpz_t n)
 
 int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
 {
-  mpz_t t, ta, tb, ra, rb, a, b, d;
+  mpz_t t, ta, tb, ra, rb, a, b, n_minus_1;
   unsigned long c = 1;
-  int k, rval = 0;
+  int bit, len, k, rval = 0;
 
   {
     int cmpr = mpz_cmp_ui(n, 2);
@@ -1101,11 +1101,12 @@ int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
   mpz_init_set_ui(ra, 1);   mpz_init_set_ui(rb, 1);
   mpz_init_set_ui(a, 1);    mpz_init_set_ui(b, 1);
   mpz_init(ta);   mpz_init(tb);
-  mpz_init(d);
-  mpz_sub_ui(d, n, 1);
+  mpz_init(n_minus_1);
+  mpz_sub_ui(n_minus_1, n, 1);
 
-  while (mpz_sgn(d)) {
-    if (mpz_odd_p(d)) {
+  len = mpz_sizeinbase(n_minus_1, 2);
+  for (bit = 0; bit < len; bit++) {
+    if ( mpz_tstbit(n_minus_1, bit) ) {
       mpz_mul(ta, ra, a);
       mpz_mul(tb, rb, b);
       mpz_add(t, ra, rb);
@@ -1118,8 +1119,7 @@ int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
       mpz_add(ra, ta, tb);
       mpz_mod(ra, ra, n);
     }
-    mpz_tdiv_q_2exp(d, d, 1);
-    if (mpz_sgn(d)) {
+    if (bit < len-1) {
       mpz_mul(t, b, b);
       mpz_mul_ui(t, t, c);
       mpz_mul(b, b, a);
@@ -1130,11 +1130,10 @@ int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
       mpz_mod(a, a, n);
     }
   }
-  mpz_sub_ui(d, n, 1);
-  if ( (mpz_cmp_ui(ra,1) == 0) && (mpz_cmp(rb, d) == 0) )
+  if ( (mpz_cmp_ui(ra,1) == 0) && (mpz_cmp(rb, n_minus_1) == 0) )
     rval = 1;
 
-  mpz_clear(d);
+  mpz_clear(n_minus_1);
   mpz_clear(ta); mpz_clear(tb);
   mpz_clear(a);  mpz_clear(b);
   mpz_clear(ra); mpz_clear(rb);
@@ -1453,17 +1452,25 @@ int _GMP_is_prime(mpz_t n)
       prob_prime = _GMP_primality_bls_nm1(n, 1 /* effort */, 0 /* cert */);
   }
 
-  /* If prob_prime is still 1, let's run some extra tests.  We could run
-   * a Frobenius test or some random-base M-R tests.  The FU test is
-   * attractive as it does not overlap with the BPSW test and gives very
-   * strong assurances.  However for small sizes it can take 6x more time
-   * than a random-base MR test (this narrows to ~2.5x at large sizes).  It
-   * also has the advantage of being deterministic.
+  /* If prob_prime is still 1, let's run some extra tests.
+   * Argument against:  Nobody has yet found a BPSW counterexample, so
+   *                    this is wasted time.  They can make a call to one of
+   *                    the other tests themselves if they care.
+   * Argument for:      is_prime() should be as correct as reasonably possible.
+   *                    They can call is_prob_prime() to skip extra tests.
    *
-   * For performance reasons, we'll use random-base MRs.
-   * Assuming we are choosing uniformly random bases and the caller cannot
-   * predict our random numbers (not guaranteed), then there is less than
-   * a 1 in 595,000 chance that a composite will pass the extra tests.
+   * Choices include:
+   *
+   *   - A number of fixed-base Miller-Rabin tests.
+   *   - A number of random-base Miller-Rabin tests.
+   *   - A Frobenius-Underwood test.
+   *   - A Frobenius-Khashin test.
+   *
+   * The Miller-Rabin tests have better performance.
+   *
+   * We will use random bases to make it more difficult to get a false
+   * result even if someone has a way to generate BPSW pseudoprimes.
+   * We use enough bases to keep the probability below 1/595,000.
    */
 
   if (prob_prime == 1) {
@@ -1475,6 +1482,7 @@ int _GMP_is_prime(mpz_t n)
     else                  ntests = 1;  /* p < .00000159 */
     prob_prime = _GMP_miller_rabin_random(n, ntests, 0);
     /* prob_prime = _GMP_is_frobenius_underwood_pseudoprime(n); */
+    /* prob_prime = _GMP_is_frobenius_khashin_pseudoprime(n); */
   }
 
   /* Using Damgård, Landrock, and Pomerance, we get upper bounds:
