@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.31';
+  $Math::Prime::Util::GMP::VERSION = '0.32';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -49,6 +49,8 @@ our @EXPORT_OK = qw(
                      ecm_factor
                      qs_factor
                      factor
+                     sigma
+                     chinese
                      moebius
                      prime_count
                      primorial
@@ -66,6 +68,7 @@ our @EXPORT_OK = qw(
                      is_power
                      znorder
                      znprimroot
+                     ramanujan_tau
                      Pi
                    );
                    # Should add:
@@ -170,7 +173,7 @@ __END__
 
 =encoding utf8
 
-=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac harmfrac harmreal stirling lucasu lucasv OpenPFGW gmpy2 nonresidue
+=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac harmfrac harmreal stirling lucasu lucasv OpenPFGW gmpy2 nonresidue chinese
 
 =head1 NAME
 
@@ -179,7 +182,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.31
+Version 0.32
 
 
 =head1 SYNOPSIS
@@ -813,6 +816,19 @@ the values satisfying Bézout's Identity.
 This corresponds to Pari's C<gcdext> function, which was renamed from
 C<bezout> out Pari 2.6.  The results will hence match L<Math::Pari/bezout>.
 
+=head2 chinese
+
+  say chinese( [14,643], [254,419], [87,733] );  # 87041638
+
+Solves a system of simultaneous congruences using the Chinese Remainder
+Theorem (with extension to non-coprime moduli).  A list of C<[a,n]> pairs
+are taken as input, each representing an equation C<x ≡ a mod n>.  If no
+solution exists, C<undef> is returned.  If a solution is returned, the
+modulus is equal to the lcm of all the given moduli (see L</lcm>.  In
+the standard case where all values of C<n> are coprime, this is just the
+product.  The C<n> values must be positive integers, while the C<a> values
+are integers.
+
 =head2 vecsum
 
 Returns the sum of all arguments, each of which must be an integer.
@@ -916,6 +932,30 @@ is a list of the smallest primitive roots, which is what this function
 produces.
 
 
+=head2 sigma
+
+  say "Sum of divisors of $n:", sigma( $n );
+  say "sigma_2($n) = ", sigma($n, 2);
+  say "Number of divisors: sigma_0($n) = ", sigma($n, 0);
+
+This function takes a positive integer as input and returns the sum of
+its divisors, including 1 and itself.  An optional second argument C<k>
+may be given, which will result in the sum of the C<k-th> powers of the
+divisors to be returned.
+
+This is known as the sigma function (see Hardy and Wright section 16.7,
+or OEIS A000203).  The API is identical to Pari/GP's C<sigma> function.
+This function is useful for calculating things like aliquot sums, abundant
+numbers, perfect numbers, etc.
+
+
+=head2 ramanujan_tau
+
+Takes a positive integer as input and returns the value of Ramanujan's tau
+function.  The result is a signed integer.
+This corresponds to Mathematica's C<RamanujanTau> function.
+
+
 =head2 valuation
 
   say "$n is divisible by 2 ", valuation($n,2), " times.";
